/*
 * Copyright (C) 2005 Cesar Kallas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * E-mail: cesarkallas@gmx.net
 * http://cesarkallas.soulivre.org
 *
 */

/**
* Funcoes do servidor
* @author Cesar Kallas - cesarkallas@gmx.net
* @since 23/11/2005
*/
package estella.servidor;
import  estella.lib.*;

import  java.io.IOException;
import  java.lang.Thread;
import  java.net.Socket;
import  java.net.SocketTimeoutException;

public class Servidor implements Runnable
{
  private ConexaoServidor conexaoServidor;
  private Thread t_conexao;
  private Log LOG;

  public Servidor()
  {
    LOG = new Log("Servidor");
    conexaoServidor = new ConexaoServidor();
  }
  
  /**
    * Inicia um novo servico de escuta em uma porta
    */
  public void run()
  {
    Socket sck_cliente;
    conexaoServidor.inicia();
    LOG.info("Iniciado @ " + conexaoServidor.porta());
    // Escuta eternamente requisicao no servidor
    while(true) {
      try {
        sck_cliente = conexaoServidor.escuta();
      }
      catch(Exception FalhaEscutarServidor)
      {
        LOG.falha(FalhaEscutarServidor.getMessage());
        return;
      }
      
      // Nova requisicao, ela eh null ? (pode ser quando o cliente conecta e desconecta)
      // Se ok, cria uma nova thread para tratar a requisicao
      if(sck_cliente != null) 
        try {
					LOG.info("Novo cliente: " + sck_cliente.getInetAddress().getHostAddress() + ":" + sck_cliente.getPort() + ":" + conexaoServidor.porta());
          t_conexao = new Thread(new Cliente(sck_cliente));
          t_conexao.start();
        }
        catch(Exception threadConexao) {
          LOG.falha(threadConexao.getMessage());
          return;
        }
    }
  }
}

/**
* Thread para tratar requisicao recebida pelo servidor
* @param sck_cliente Socket com a conexao do cliente
* @since 23/11/2005
* @author Cesar Kallas - cesarkallas at gmx . net
*/
class Cliente implements Runnable
{
  private Socket sck_cliente;
  private Conexao conexao;
  private Log LOG;
  private Constantes CON;
	private Descritor descritor;
	private String mensagem;
	private Dados dados;
	private Thread t_trataDescritor;
		
  public Cliente(Socket sck_cliente)
  {
    LOG = new Log("Servidor");
    CON = new Constantes();
    dados = new Dados();
		conexao = new Conexao(sck_cliente);
  }

  public void run()
  {
		// Trata pedido de conexao do cliente
		try {
			conexao.inicia();
			conexao.setTimeout(CON.SOCKET_SERVER_TIMEOUT);
			mensagem = conexao.recebe();
			if(mensagem.indexOf(CON.GNUTELLA) == -1)
				LOG.falha("Cabecalho de autenticacao invalido - " + conexao.status());
			else
				if(conexao.envia(CON.GNUTELLA_OK + "\n\n")) {
					trataCliente();
				}
			else
				LOG.falha("ao enviar resposta de pedido de conexao para " + conexao.status());
		}
		catch(Exception FalhaAoConectar)
		{}

		try {
			conexao.fecha();
		}
		catch(Exception FalhaDesconectarCliente)
		{}
		return;
  }
		
	public void trataCliente()
	{
		conexao.setTimeout(0);

		// Cliente conectado, tratar descritores do cliente
		while(conexao.status() != null) {
			try {
				mensagem = new String(conexao.recebe());
			}
			catch(Exception ReceberDescritor)
			{
				LOG.falha(ReceberDescritor.getMessage());
				return;
			}
			descritor = new Descritor(mensagem);
			t_trataDescritor = new Thread(new trataDescritor(descritor, conexao));
			t_trataDescritor.start();
		}
		LOG.info("Cliente desconectado");
		return;
	}
}

class trataDescritor implements Runnable
{
	private Descritor descritor;
	private Log LOG;
	private Constantes CON;
	private Conexao conexao;
		
	public trataDescritor(Descritor descritor, Conexao conexao)
	{
		this.descritor = descritor;
		this.conexao = conexao;
		CON = new Constantes();
		LOG = new Log();
	}
	
	public void run()
	{
		// Tamanho minimo de descritor 20 (id + ttl + hops + payload length)
		if(descritor.retorna().length < 20)
			return;
		Dados.listaDescritores.adiciona(descritor);
		switch(descritor.tipo())
		{
			case (byte)00000000: ping(); break;
			case (byte)00000001: pong(); break;
			case (byte)10000000: query(); break;
			case (byte)10000001: queryhits(); break;
		}
	}
	
	private void ping()
	{
		LOG.info("ping de " + conexao.status());
	}
	
	private void pong()
	{
		LOG.info("pong de " + conexao.status());
	}
	
	private void query()
	{
		LOG.info("query de " + conexao.status());
	}
	
	private void queryhits()
	{
		LOG.info("queryhits de " + conexao.status());
		
	}
}
